from PyQt5 import QtCore
from PyQt5.QtWidgets import QFileDialog, QLineEdit, QHBoxLayout, QVBoxLayout, QWidget, QFileDialog, QCheckBox
from niaaml_gui.widgets.base_main_widget import BaseMainWidget
from niaaml_gui.windows import ProcessWindow
from niaaml_gui.process_window_data import ProcessWindowData
import qtawesome as qta

class UsePipelineWidget(BaseMainWidget):
    def __init__(self, parent, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
    
        vBoxLayout = QVBoxLayout(self._parent)
        vBoxLayout.setAlignment(QtCore.Qt.AlignTop)

        selectPPLNFileBar = QHBoxLayout(self._parent)
        selectPPLNFileBar.setSpacing(0)
        fNameLine1 = QLineEdit(self._parent)
        fNameLine1.setObjectName('pplnFile')
        fNameLine1.setPlaceholderText('Select a pipeline file...')
        fNameLine1.setReadOnly(True)
        font = fNameLine1.font()
        font.setPointSize(12)
        fNameLine1.setFont(font)
        selectPPLNFileBar.addWidget(fNameLine1)
        selectPPLNFileBar.addWidget(self._createButton('Select file', self.__openPPLNFile))

        fileLayout = QHBoxLayout(self._parent)

        selectFileBar = QHBoxLayout(self._parent)
        selectFileBar.setSpacing(0)
        selectFileBar.setContentsMargins(0, 0, 5, 0)
        fNameLine = QLineEdit(self._parent)
        fNameLine.setObjectName('csvFile')
        fNameLine.setPlaceholderText('Select a CSV file with features...')
        fNameLine.setReadOnly(True)
        fNameLine.setFont(font)
        selectFileBar.addWidget(fNameLine)
        editBtn = self._createButton(None, self._editCSVFile, 'editCSVButton', qta.icon('fa5.edit'))
        editBtn.setEnabled(False)
        selectFileBar.addWidget(editBtn)
        selectFileBar.addWidget(self._createButton('Select file', self._openCSVFile))

        checkBox = QCheckBox('CSV has header')
        checkBox.setObjectName('csv')
        checkBox.setFont(font)

        fileLayout.addItem(selectFileBar)
        fileLayout.addWidget(checkBox)

        confirmBar = QHBoxLayout(self._parent)
        confirmBar.addStretch()
        confirmBar.addWidget(self._createButton('Run', self.__runPipeline))

        vBoxLayout.addItem(selectPPLNFileBar)
        vBoxLayout.addItem(fileLayout)
        vBoxLayout.addItem(confirmBar)

        self.setLayout(vBoxLayout)

    def __openPPLNFile(self):
        fname = QFileDialog.getOpenFileName(parent=self._parent, caption='Select pipeline File', filter='PPLN files (*.PPLN)')
        self.findChild(QLineEdit, 'pplnFile').setText(fname[0])
    
    def __runPipeline(self):
        err = ''

        pplnSrc = self.findChild(QLineEdit, 'pplnFile').text()
        if self._isNoneOrWhiteSpace(pplnSrc):
            err += 'Select pipeline file.\n'

        csvSrc = self.findChild(QLineEdit, 'csvFile').text()
        if self._isNoneOrWhiteSpace(csvSrc):
            err += 'Select CSV file with features.\n'

        if not self._isNoneOrWhiteSpace(err):
            self._parent.errorMessage.setText(err)
            self._parent.errorMessage.show()
            return

        self._processWindow = ProcessWindow(
            self._parent,
            ProcessWindowData(
                False,
                csvSrc=csvSrc,
                csvHasHeader=self.findChild(QCheckBox, 'csv').isChecked(),
                pipelineSrc=pplnSrc
                )
            )
        self._processWindow.show()


from PyQt5 import QtCore
from niaaml.data import CSVDataReader
from niaaml import PipelineOptimizer
import os
import sys

class OptimizeThread(QtCore.QThread):

    optimized = QtCore.pyqtSignal(object)
    progress = QtCore.pyqtSignal(object)

    def __init__(self, data):
        QtCore.QThread.__init__(self)
        self.__data = data
    
    def run(self):
        dataReader = CSVDataReader(src=self.__data.csvSrc, has_header=self.__data.csvHasHeader)
        optimizer = PipelineOptimizer(
            data=dataReader,
            feature_selection_algorithms=self.__data.fsas,
            feature_transform_algorithm=self.__data.ftas,
            classifiers=self.__data.classifiers,
            categorical_features_encoder=self.__data.encoder,
            imputer=self.__data.imputer
        )
        optimizer._PipelineOptimizer__logger = HackyLogger(self.progress.emit)

        if self.__data.isOptimization is True:
            pipeline = optimizer.run(self.__data.fitnessFunctionName, self.__data.popSize, self.__data.popSizeInner, self.__data.numEvals, self.__data.numEvalsInner, self.__data.optAlgName, self.__data.optAlgInnerName)
        else:
            pipeline = optimizer.run_v1(self.__data.fitnessFunctionName, self.__data.popSize, self.__data.numEvals, self.__data.optAlgName)

        pipeline.export(os.path.join(self.__data.outputFolder, 'niaamlGUIoutput'))
        pipeline.export_text(os.path.join(self.__data.outputFolder, 'niaamlGUIoutput'))
        self.optimized.emit(pipeline.to_string())

class HackyLogger:
    def __init__(self, emit_func,  **kwargs):
        self.__emit = emit_func

    def log_progress(self, text):
        self.__emit(text)

    def log_pipeline(self, text):
        return

    def log_optimization_error(self, text):
        return



from PyQt5 import QtCore
from niaaml.data import CSVDataReader
from niaaml import Pipeline

class RunThread(QtCore.QThread):

    ran = QtCore.pyqtSignal(object)

    def __init__(self, data):
        QtCore.QThread.__init__(self)
        self.__data = data
    
    def run(self):
        dataReader = CSVDataReader(src=self.__data.csvSrc, contains_classes=False, has_header=self.__data.csvHasHeader)
        pipeline = Pipeline.load(self.__data.pipelineSrc)
        predictions = pipeline.run(dataReader.get_x())
        self.ran.emit(str(predictions))


class ProcessWindowData:
    def __init__(
        self,
        isOptimization,
        csvSrc = None,
        csvHasHeader = None,
        encoder = None,
        imputer = None,
        optAlgName = None,
        optAlgInnerName = None,
        popSize = None,
        popSizeInner = None,
        numEvals = None,
        numEvalsInner = None,
        fsas = None,
        ftas = None,
        classifiers = None,
        fitnessFunctionName = None,
        outputFolder = None,
        pipelineSrc = None):
        self.isOptimization = isOptimization
        self.csvSrc = csvSrc
        self.csvHasHeader = csvHasHeader
        self.encoder = encoder
        self.imputer = imputer
        self.optAlgName = optAlgName
        self.optAlgInnerName = optAlgInnerName
        self.popSize = popSize
        self.popSizeInner = popSizeInner
        self.numEvals = numEvals
        self.numEvalsInner = numEvalsInner
        self.fsas = fsas if fsas is not None and len(fsas) > 0 else None
        self.ftas = ftas if ftas is not None and len(ftas) > 0 else None
        self.classifiers = classifiers
        self.fitnessFunctionName = fitnessFunctionName
        self.outputFolder = outputFolder
        self.pipelineSrc = pipelineSrc


from niaaml_gui import widgets
from niaaml_gui import windows
from niaaml_gui.main import MainAppWindow
from niaaml_gui.process_window_data import ProcessWindowData
from niaaml_gui.progress_bar import ProgressBar

__all__ = [
    'widgets',
    'windows',
    'ProcessWindowData',
    'ProgressBar',
    'MainAppWindow'
]
__version__ = '0.1.13'



from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QMainWindow, QWidget, QHBoxLayout, QTableView, QMessageBox, QVBoxLayout, QPushButton
from PyQt5.QtCore import QSize
import csv
import qtawesome as qta

class CSVEditorWindow(QMainWindow):
    def __init__(self, src):
        super(CSVEditorWindow, self).__init__()
        self.setMinimumSize(QSize(640, 480))

        self.__src = src
        self.__table = QTableView(self)
        self.__model = QtGui.QStandardItemModel(self)
        self.__table.setModel(self.__model)

        try:
            with open(src, 'r') as f:
                for row in csv.reader(f):
                    items = [
                        QStandardItem(field) for field in row
                    ]
                    self.__model.appendRow(items)
        except:
            self.__errorMessage = QMessageBox(QMessageBox.Critical, 'Error', 'File could not be read.')
            self.__errorMessage.show()
            return

        centralWidget = QWidget(self)
        layout = QVBoxLayout(centralWidget)
        
        toolBar = QHBoxLayout(centralWidget)
        toolBar.setAlignment(QtCore.Qt.AlignLeft)
        saveBtn = self.__createButton(None, self.__save, None, qta.icon('fa5.save'))
        toolBar.addWidget(saveBtn)

        layout.addItem(toolBar)
        layout.addWidget(self.__table)

        centralWidget.setLayout(layout)
        self.setCentralWidget(centralWidget)

    def __createButton(self, text, callback = None, objectName = None, icon = None):
        btn = QPushButton(self)
        btn.setText(text)
        font = btn.font()
        font.setPointSize(12)
        btn.setFont(font)

        if callback is not None:
            btn.clicked.connect(callback)
        
        if objectName is not None:
            btn.setObjectName(objectName)

        if icon is not None:
            btn.setIcon(icon)
            btn.setIconSize(QtCore.QSize(21, 21))

        return btn
    
    def __save(self):
        self.__writeCsv(self.__src)
    
    def keyPressEvent(self, e):
        if e.key() == QtCore.Qt.Key_Delete or e.key() == QtCore.Qt.Key_Backspace:
            rows = self.__table.selectionModel().selectedRows()
            for index in rows:
                self.__model.removeRow(index.row())

            cols = self.__table.selectionModel().selectedColumns()
            for index in cols:
                self.__model.removeColumn(index.column())
            
            cells = self.__table.selectedIndexes()
            for index in cells:
                if self.__model.item(index.row(), index.column()) is not None:
                    self.__model.item(index.row(), index.column()).setText(None)

    def __writeCsv(self, fileName):
        try:
            with open(fileName, 'w', newline='') as fileOutput:
                writer = csv.writer(fileOutput)
                for rowNumber in range(self.__model.rowCount()):
                    fields = [
                        self.__model.data(
                            self.__model.index(rowNumber, columnNumber),
                            QtCore.Qt.DisplayRole
                        )
                        for columnNumber in range(self.__model.columnCount())
                    ]
                    writer.writerow(fields)
        except:
            self.__errorMessage = QMessageBox(QMessageBox.Critical, 'Error', 'File could not be saved.')
            self.__errorMessage.show()
            return



from PyQt5.QtWidgets import QComboBox, QLineEdit, QLabel, QHBoxLayout, QVBoxLayout, QGridLayout, QTabWidget, QFileDialog, QCheckBox, QPushButton
from PyQt5 import QtGui
from PyQt5.QtGui import QIcon
from PyQt5 import QtCore
from niapy.util.factory import _algorithm_options
from niaaml_gui.widgets.list_widget_custom import ListWidgetCustom
from niaaml_gui.widgets.base_main_widget import BaseMainWidget
from niaaml_gui.windows import ProcessWindow
from niaaml_gui.process_window_data import ProcessWindowData
from niaaml.classifiers import ClassifierFactory
from niaaml.preprocessing.feature_selection import FeatureSelectionAlgorithmFactory
from niaaml.preprocessing.feature_transform import FeatureTransformAlgorithmFactory
from niaaml.fitness import FitnessFactory
from niaaml.preprocessing.encoding import EncoderFactory
from niaaml.preprocessing.imputation import ImputerFactory
import qtawesome as qta

class OptimizationWidget(BaseMainWidget):
    __niaamlFeatureSelectionAlgorithms = FeatureSelectionAlgorithmFactory().get_name_to_classname_mapping()
    __niaamlFeatureTransformAlgorithms = FeatureTransformAlgorithmFactory().get_name_to_classname_mapping()
    __niaamlClassifiers = ClassifierFactory().get_name_to_classname_mapping()
    __niaamlFitnessFunctions = FitnessFactory().get_name_to_classname_mapping()
    __niaamlEncoders = EncoderFactory().get_name_to_classname_mapping()
    __niaamlImputers = ImputerFactory().get_name_to_classname_mapping()
    __niapyAlgorithmsList = list(_algorithm_options().keys())
    __niaamlFeatureSelectionAlgorithmsList = list(__niaamlFeatureSelectionAlgorithms.keys())
    __niaamlFeatureTransformAlgorithmsList = list(__niaamlFeatureTransformAlgorithms.keys())
    __niaamlClassifiersList = list(__niaamlClassifiers.keys())
    __niaamlFitnessFunctionsList = list(__niaamlFitnessFunctions.keys())
    __niaamlEncodersList = list(__niaamlEncoders.keys())
    __niaamlImputersList = list(__niaamlImputers.keys())

    def __init__(self, parent, is_v1 = False, *args, **kwargs):
        self.__niapyAlgorithmsList.sort()
        self.__niaamlFeatureSelectionAlgorithmsList.sort()
        self.__niaamlFeatureTransformAlgorithmsList.sort()
        self.__niaamlClassifiersList.sort()
        self.__is_v1 = is_v1
        super().__init__(parent, *args, **kwargs)

        fileLayout = QHBoxLayout(self._parent)

        selectFileBar = QHBoxLayout(self._parent)
        selectFileBar.setSpacing(0)
        selectFileBar.setContentsMargins(0, 5, 5, 5)
        fNameLine = QLineEdit(self._parent)
        fNameLine.setObjectName('csvFile')
        fNameLine.setPlaceholderText('Select a CSV dataset file...')
        fNameLine.setReadOnly(True)
        font = fNameLine.font()
        font.setPointSize(12)
        fNameLine.setFont(font)
        selectFileBar.addWidget(fNameLine)
        editBtn = self._createButton(None, self._editCSVFile, 'editCSVButton', qta.icon('fa5.edit'))
        editBtn.setEnabled(False)
        selectFileBar.addWidget(editBtn)
        selectFileBar.addWidget(self._createButton('Select file', self._openCSVFile))

        checkBox = QCheckBox('CSV has header')
        checkBox.setObjectName('csv')
        checkBox.setFont(font)

        fileLayout.addItem(selectFileBar)
        fileLayout.addWidget(checkBox)

        encoders = self.__createComboBox('Categorical features\' encoder:', self.__niaamlEncodersList, 'encoders')
        imputers = self.__createComboBox('Missing features\' imputer:', self.__niaamlImputersList, 'imputers')

        hBoxLayout = QHBoxLayout(self._parent)
        hBoxLayout.setContentsMargins(0, 0, 0, 0)

        vBoxLayout = QVBoxLayout(self._parent)
        vBoxLayout.setContentsMargins(0, 34, 0, 0)

        h1BoxLayout = QHBoxLayout(self._parent)
        h1BoxLayout.setContentsMargins(0, 0, 0, 0)
        
        fsasBox = self.__createGridLayoutBox((0, 5, 3, 5), True)
        fsasList = self.__createListWidget([], None, 'fsasList')
        fsasBox.addWidget(fsasList)
        h1BoxLayout.addItem(fsasBox)

        ftasBox = self.__createGridLayoutBox((3, 5, 3, 5), True)
        ftasList = self.__createListWidget([], None, 'ftasList')
        ftasBox.addWidget(ftasList)
        h1BoxLayout.addItem(ftasBox)

        classifiers = self.__createGridLayoutBox((3, 5, 5, 5), True)
        classifiersList = self.__createListWidget([], None, 'classifiersList')
        classifiers.addWidget(classifiersList)
        h1BoxLayout.addItem(classifiers)

        settingsBox = self.__createGridLayoutBox((0, 0, 5, 5), False, 'transparent')
        settingsBox.setVerticalSpacing(10)

        optAlgosLabel = 'Optimization Algorithm (components selection):' if not self.__is_v1 else 'Optimization Algorithm:'
        optAlgos = self.__createComboBox(optAlgosLabel, self.__niapyAlgorithmsList, 'optAlgos')

        optAlgosInner = self.__createComboBox('Optimization Algorithm (parameter tuning) - same as first if not selected:', [*['None'], *self.__niapyAlgorithmsList], 'optAlgosInner')

        validator = QtGui.QRegExpValidator(QtCore.QRegExp('[1-9][0-9]*'))

        popSizeLabel = 'Population size (components selection):' if not self.__is_v1 else 'Population size:'
        popSize = self.__createTextInput(popSizeLabel, 'popSize', validator)

        popSizeInner = self.__createTextInput('Population size (parameter tuning):', 'popSizeInner', validator)

        numEvalsLabel = 'Number of evaluations (components selection):' if not self.__is_v1 else 'Number of evaluations'
        numEvals = self.__createTextInput(numEvalsLabel, 'numEvals', validator)

        numEvalsInner = self.__createTextInput('Number of evaluations (parameter tuning):', 'numEvalsInner', validator)

        fitFuncs = self.__createComboBox('Fitness Function:', self.__niaamlFitnessFunctionsList, 'fitFuncs')

        selectOutputFolderBar = QHBoxLayout(self._parent)
        selectOutputFolderBar.setSpacing(0)
        foNameLine = QLineEdit(self._parent)
        foNameLine.setObjectName('outputFolder')
        foNameLine.setPlaceholderText('Select pipeline output folder...')
        foNameLine.setReadOnly(True)
        font = foNameLine.font()
        font.setPointSize(12)
        foNameLine.setFont(font)
        selectOutputFolderBar.addWidget(foNameLine)
        selectOutputFolderBar.addWidget(self._createButton('Select folder', self.__selectDirectory))

        settingsBox.addItem(optAlgos)
        if not self.__is_v1:
            settingsBox.addItem(optAlgosInner)
        settingsBox.addItem(popSize)
        if not self.__is_v1:
            settingsBox.addItem(popSizeInner)
        settingsBox.addItem(numEvals)
        if not self.__is_v1:
            settingsBox.addItem(numEvalsInner)
        settingsBox.addItem(fitFuncs)
        settingsBox.addItem(selectOutputFolderBar)

        confirmBar = QHBoxLayout(self._parent)
        confirmBar.setContentsMargins(5, 5, 5, 5)
        confirmBar.addStretch()
        confirmBar.addWidget(self._createButton('Start optimization', self.__runOptimize))

        vBoxLayout.addItem(fileLayout)
        vBoxLayout.addItem(encoders)
        vBoxLayout.addItem(imputers)
        vBoxLayout.addItem(h1BoxLayout)
        vBoxLayout.addItem(settingsBox)
        vBoxLayout.addItem(confirmBar)

        exploreBox = self.__createGridLayoutBox((0, 0, 0, 0), False)
        exploreBox.addWidget(self.__createTabs(fsasList, ftasList, classifiersList))

        hBoxLayout.addItem(exploreBox)
        hBoxLayout.addItem(vBoxLayout)
        hBoxLayout.setStretchFactor(exploreBox, 1)
        hBoxLayout.setStretchFactor(vBoxLayout, 2)

        self.setLayout(hBoxLayout)
    
    def __createComboBox(self, label, items, name):
        comboBox = QVBoxLayout()
        comboBox.setSpacing(5)
        label = QLabel(label, self._parent)
        font = label.font()
        font.setPointSize(12)
        label.setFont(font)
        cb = QComboBox()
        cb.setObjectName(name)
        cb.setFont(font)
        for k in items:
            cb.addItem(k)
        comboBox.addWidget(label)
        comboBox.addWidget(cb)
        return comboBox
    
    def __createTextInput(self, label, name, validator=None):
        textBox = QVBoxLayout()
        textBox.setSpacing(5)
        label = QLabel(label, self._parent)
        font = label.font()
        font.setPointSize(12)
        label.setFont(font)
        tb = QLineEdit(self._parent)
        tb.setObjectName(name)
        tb.setFont(font)
        textBox.addWidget(label)
        textBox.addWidget(tb)

        if validator is not None:
            tb.setValidator(validator)

        return textBox

    def __createGridLayoutBox(self, tupleMargins, visibleBorder, background_color = '#fff'):
        l = QGridLayout()
        l.setContentsMargins(*tupleMargins)
        return l
    
    def __createListWidget(self, items, targetBox = None, name = None):
        listWidget = ListWidgetCustom(items, targetBox, name)
        font = listWidget.font()
        font.setPointSize(12)
        listWidget.setFont(font)
        return listWidget
    
    def __createTabs(self, fsasList, ftasList, classifiersList):
        tabs = QTabWidget(self._parent)

        fsas = self.__createListWidget(self.__niaamlFeatureSelectionAlgorithmsList, fsasList)
        fsasList.setTarget(fsas)
        tabs.addTab(fsas, 'Feature Selection Algorithms')

        ftas = self.__createListWidget(self.__niaamlFeatureTransformAlgorithmsList, ftasList)
        ftasList.setTarget(ftas)
        tabs.addTab(ftas, 'Feature Selection Algorithms')

        clas = self.__createListWidget(self.__niaamlClassifiersList, classifiersList)
        classifiersList.setTarget(clas)
        tabs.addTab(clas, 'Classifiers')

        font = tabs.font()
        font.setPointSize(10)
        tabs.setFont(font)
        tabs.setStyleSheet("QTabBar::tab { height: 40px; }")
        return tabs
    
    def __selectDirectory(self):
        fname = str(QFileDialog.getExistingDirectory(parent=self._parent, caption='Select Directory'))
        self.findChild(QLineEdit, 'outputFolder').setText(fname)
    
    def __runOptimize(self):
        err = ''

        csvSrc = self.findChild(QLineEdit, 'csvFile').text()
        if self._isNoneOrWhiteSpace(csvSrc):
            err += 'Select CSV dataset file.\n'
        
        encoderName = self.__niaamlEncoders[str(self.findChild(QComboBox, 'encoders').currentText())]
        imputerName = self.__niaamlImputers[str(self.findChild(QComboBox, 'imputers').currentText())]

        optAlgName = str(self.findChild(QComboBox, 'optAlgos').currentText())

        if not self.__is_v1:
            optAlgInnerName = str(self.findChild(QComboBox, 'optAlgosInner').currentText())
            if optAlgInnerName == 'None':
                optAlgInnerName = optAlgName

        popSize = self.findChild(QLineEdit, 'popSize').text()
        if self._isNoneOrWhiteSpace(popSize):
            err += 'Select population size.\n'
        else:
            try:
                popSize = int(popSize)
            except:
                err += 'Invalid population size value.\n'

        if not self.__is_v1:
            popSizeInner = self.findChild(QLineEdit, 'popSizeInner').text()
            if self._isNoneOrWhiteSpace(popSizeInner):
                err += 'Select inner population size.\n'
            else:
                try:
                    popSizeInner = int(popSizeInner)
                except:
                    err += 'Invalid inner population size value.\n'

        numEvals = self.findChild(QLineEdit, 'numEvals').text()
        if self._isNoneOrWhiteSpace(numEvals):
            err += 'Select number of evaluations.\n'
        else:
            try:
                numEvals = int(numEvals)
            except:
                err += 'Invalid number of evaluations.\n'

        if not self.__is_v1:
            numEvalsInner = self.findChild(QLineEdit, 'numEvalsInner').text()
            if self._isNoneOrWhiteSpace(numEvalsInner):
                err += 'Select number of inner evaluations.\n'
            else:
                try:
                    numEvalsInner = int(numEvalsInner)
                except:
                    err += 'Invalid number of inner evaluations.\n'
        
        fsasList = self.findChild(ListWidgetCustom, 'fsasList')
        fsas = [self.__niaamlFeatureSelectionAlgorithms[fsasList.item(i).text()] for i in range(fsasList.count())]

        ftasList = self.findChild(ListWidgetCustom, 'ftasList')
        ftas = [self.__niaamlFeatureTransformAlgorithms[ftasList.item(i).text()] for i in range(ftasList.count())]

        clsList = self.findChild(ListWidgetCustom, 'classifiersList')
        classifiers = [self.__niaamlClassifiers[clsList.item(i).text()] for i in range(clsList.count())]
        if len(classifiers) == 0:
            err += 'Select at least one classifier.\n'
        
        fitnessFunctionName = self.__niaamlFitnessFunctions[str(self.findChild(QComboBox, 'fitFuncs').currentText())]

        outputFolder = self.findChild(QLineEdit, 'outputFolder').text()
        if self._isNoneOrWhiteSpace(outputFolder):
            err += 'Select an output directory.\n'
        
        if not self._isNoneOrWhiteSpace(err):
            self._parent.errorMessage.setText(err)
            self._parent.errorMessage.show()
            return
        
        if not self.__is_v1:
            self._processWindow = ProcessWindow(
                self._parent,
                ProcessWindowData(
                    True,
                    csvSrc,
                    self.findChild(QCheckBox, 'csv').isChecked(),
                    encoderName,
                    imputerName,
                    optAlgName,
                    optAlgInnerName,
                    popSize,
                    popSizeInner,
                    numEvals,
                    numEvalsInner,
                    fsas,
                    ftas,
                    classifiers,
                    fitnessFunctionName,
                    outputFolder
                    )
                )
        else:
            self._processWindow = ProcessWindow(
                self._parent,
                ProcessWindowData(
                    'v1',
                    csvSrc,
                    self.findChild(QCheckBox, 'csv').isChecked(),
                    encoderName,
                    imputerName,
                    optAlgName=optAlgName,
                    popSize=popSize,
                    numEvals=numEvals,
                    fsas=fsas,
                    ftas=ftas,
                    classifiers=classifiers,
                    fitnessFunctionName=fitnessFunctionName,
                    outputFolder=outputFolder
                    )
                )

        self._processWindow.show()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['niaaml_gui',
 'niaaml_gui.widgets',
 'niaaml_gui.windows',
 'niaaml_gui.windows.threads']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.15.0,<6.0.0',
 'QtAwesome>=1.0.2,<2.0.0',
 'niaaml>=1.1.11,<2.0.0',
 'niapy>=2.0.2,<3.0.0']

entry_points = \
{'console_scripts': ['NiaAML-GUI = niaaml_gui.main:run']}

setup_kwargs = {
    'name': 'niaaml-gui',
    'version': '0.1.13',
    'description': 'GUI for NiaAML Python package',
    'long_description': '',
    'author': 'Luka Pečnik',
    'author_email': 'lukapecnik96@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/lukapecnik/NiaAML-GUI',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from PyQt5 import QtCore
from PyQt5.QtWidgets import QProgressBar

class ProgressBar(QProgressBar):
    def __init__(self):
        super(ProgressBar, self).__init__()
        self.setTextVisible(True)
        self.setMaximum(100)
        self.setAlignment(QtCore.Qt.AlignCenter)


from PyQt5 import QtCore
from PyQt5.QtWidgets import QMainWindow, QVBoxLayout, QWidget, QPlainTextEdit, QPushButton, QHBoxLayout
from PyQt5.QtCore import QSize
from niaaml_gui.progress_bar import ProgressBar
from niaaml_gui.windows.threads import OptimizeThread, RunThread
import copy

class ProcessWindow(QMainWindow):
    def __init__(self, parent, data):
        super(ProcessWindow, self).__init__(parent)
        self.setMinimumSize(QSize(640, 480))

        centralWidget = QWidget(self)
        layout = QVBoxLayout(centralWidget)
        layout.setAlignment(QtCore.Qt.AlignTop)
        self.__progressBar = ProgressBar()
        layout.addWidget(self.__progressBar)

        self.__textArea = QPlainTextEdit(parent=self)
        self.__textArea.setReadOnly(True)
        layout.addWidget(self.__textArea)

        confirmBar = QHBoxLayout(self)
        confirmBar.addStretch()

        self.__btn = QPushButton(self)
        self.__btn.setText('Cancel')
        font = self.__btn.font()
        font.setPointSize(12)
        self.__btn.setFont(font)
        self.__btn.clicked.connect(self.cancelClose)
        confirmBar.addWidget(self.__btn)

        layout.addItem(confirmBar)

        centralWidget.setLayout(layout)
        self.setCentralWidget(centralWidget)

        self.__data = copy.deepcopy(data)

        if self.__data.isOptimization is True or self.__data.isOptimization == 'v1':
            self.__progressBar.setMaximum(100)
            self.__currentEvals = 0
            self.__totalEvals = data.numEvals * data.numEvalsInner if data.isOptimization is True else data.numEvals
            optimizer = OptimizeThread(self.__data)
            optimizer.optimized.connect(self.onOptimizationComplete)
            optimizer.progress.connect(self.onOptimizationProgress)
            self.__runningThread = optimizer
            optimizer.start()
            self.__textArea.appendPlainText('Pipeline optimization running...\n')
        else:
            self.__progressBar.setMaximum(0)
            self.__progressBar.setTextVisible(False)
            runner = RunThread(self.__data)
            runner.ran.connect(self.onRunComplete)
            self.__runningThread = runner
            runner.start()
            self.__textArea.appendPlainText('Pipeline running...\n')
    
    def cancelClose(self):
        self.close()
        try:
            self.__runningThread.terminate()
        except:
            return
    
    def onOptimizationComplete(self, data):
        self.__progressBar.setValue(100)
        self.__textArea.appendPlainText(data + '\n')
        self.__textArea.appendPlainText('Pipeline optimization complete.')
        self.__textArea.appendPlainText('Results exported to: ' + self.__data.outputFolder)
        self.__btn.setText('Close')

    def onOptimizationProgress(self, data):
        if data.startswith('Evaluation'):
            self.__currentEvals += 1
            val = int((self.__currentEvals / self.__totalEvals) * 100)
            self.__progressBar.setValue(val)

    def onRunComplete(self, data):
        self.__progressBar.setMaximum(100)
        self.__progressBar.setValue(100)
        self.__textArea.appendPlainText('Predictions: ' + data + '\n')
        self.__textArea.appendPlainText('Pipeline run complete.')
        self.__btn.setText('Close')




import sys
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import qApp, QMainWindow, QMessageBox, QAction
from PyQt5.QtCore import QSize
from niaaml_gui.widgets import OptimizationWidget, UsePipelineWidget

class WriteStream(object):
    def __init__(self,queue):
        self.queue = queue

    def write(self, text):
        self.queue.append(text)

class MainAppWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.setMinimumSize(QSize(1024, 768))
        self.setWindowTitle('NiaAML - GUI')

        menuBar = self.menuBar()
        menuBar.setNativeMenuBar(False)
        fileMenu = menuBar.addMenu('&File')
        exitAction = QAction(text='Exit', parent=self)
        exitAction.triggered.connect(qApp.quit)
        newPipelineAction = QAction(text='New Pipeline', parent=self)
        newPipelineAction.triggered.connect(self.__setOptimizationView)
        newPipelineActionV1 = QAction(text='New Pipeline V1', parent=self)
        newPipelineActionV1.triggered.connect(self.__setOptimizationV1View)
        useExistingPipelineAction = QAction(text='Use Existing Pipeline', parent=self)
        useExistingPipelineAction.triggered.connect(self.__setUsePipelineView)

        fileMenu.addAction(newPipelineAction)
        fileMenu.addAction(newPipelineActionV1)
        fileMenu.addAction(useExistingPipelineAction)
        fileMenu.addAction(exitAction)

        self.setCentralWidget(OptimizationWidget(self))

        self.errorMessage = QMessageBox()
        self.errorMessage.setIcon(QMessageBox.Critical)
        self.errorMessage.setWindowTitle('Error')
        self.errorMessage.setStandardButtons(QMessageBox.Ok)

    def __setOptimizationView(self):
        self.setCentralWidget(OptimizationWidget(self))

    def __setOptimizationV1View(self):
        self.setCentralWidget(OptimizationWidget(self, True))

    def __setUsePipelineView(self):
        self.setCentralWidget(UsePipelineWidget(self))

def run():
    app = QtWidgets.QApplication(sys.argv)
    app.setStyle('Fusion')
    mainWin = MainAppWindow()
    mainWin.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    run()



from niaaml_gui.widgets.list_widget_custom import ListWidgetCustom
from niaaml_gui.widgets.optimization_widget import OptimizationWidget
from niaaml_gui.widgets.use_pipeline_widget import UsePipelineWidget

__all__ = [
    'ListWidgetCustom',
    'OptimizationWidget',
    'UsePipelineWidget'
]


from PyQt5.QtWidgets import QWidget, QPushButton, QLineEdit, QFileDialog
from niaaml_gui.windows import CSVEditorWindow
from PyQt5 import QtCore

class BaseMainWidget(QWidget):
    def __init__(self, parent, *args, **kwargs):
        super(QWidget, self).__init__(*args, **kwargs)
        self._parent = parent
        self._processWindow = None
        self.__csvEditWindow = None

    def _createButton(self, text, callback = None, objectName = None, icon = None):
        btn = QPushButton(self._parent)
        btn.setText(text)
        font = btn.font()
        font.setPointSize(12)
        btn.setFont(font)

        if callback is not None:
            btn.clicked.connect(callback)
        
        if objectName is not None:
            btn.setObjectName(objectName)

        if icon is not None:
            btn.setIcon(icon)
            btn.setIconSize(QtCore.QSize(21, 21))

        return btn

    def _isNoneOrWhiteSpace(self, text):
        if text is None or text.isspace() or len(text) == 0:
            return True

    def _openCSVFile(self):
        fname = QFileDialog.getOpenFileName(parent=self._parent, caption='Select CSV File', filter='CSV files (*.csv)')
        self.findChild(QLineEdit, 'csvFile').setText(fname[0])
        self.findChild(QPushButton, 'editCSVButton').setEnabled(True)

    def _editCSVFile(self):
        src = self.findChild(QLineEdit, 'csvFile').text()
        self.__csvEditWindow = CSVEditorWindow(src)
        self.__csvEditWindow.show()